/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *     																	   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "LuaSearch.h"
#include <QDebug>
const char LuaSearch::className[] = "LuaSearch";

const Luna<LuaSearch>::RegType
        LuaSearch::Register[] = { { "search",       &LuaSearch::search       },
                                  { "next",         &LuaSearch::next         },
                                  { "previous",     &LuaSearch::previous     },
                                  { "addItems",     &LuaSearch::addItems     },
                                  { "selectAll",    &LuaSearch::selectAll    },
                                  { "searchTable",  &LuaSearch::searchTable  },
                                  { "numberOfHits", &LuaSearch::numberOfHits },
                                  { "hasNext",      &LuaSearch::hasNext      },
                                  { "hasPrevious",  &LuaSearch::hasPrevious  },
                                  { 0, 0 } };
/**
 * \brief Sets the SearchController reference to the object which is registered
 *        in Lua.
 *
 * \param L The lua_State to use.
 */
LuaSearch::LuaSearch ( lua_State* L ) : LuaWrapper(L,"SearchController")
{
   	searchControl_ = (SearchController*) lua_touserdata ( L, 1 );
}

/**
 * \brief Emtpy destructor (for Luna).
 */
LuaSearch::~LuaSearch ()
{
}

/**
 * \brief Search for letters.
 *
 * This method searches for letters in the visible table. It takes two
 * arguments from the Lua interpreter. The first argument is the search
 * string or regular expression. The second argument are the search flags.
 * The search flags are:<br>
 * 'c' Codepoint<br>
 * 'w' Collation<br>
 * 'd' Comment<br>
 * 'r' Regular Expression<br>
 * 'D' Case sensitive<br>
 * 'l1-4' Collation level 1-4<br>
 * If the case sensitive flag is set, the comment flag is set, too. And
 * if a collation level flag is set, the collation flag is set, too.
 * After the search() method from SearchController is called, the next() method is
 * called to select the first found letter.
 *
 * \param L The lua_State to use.
 * \return Returns number of results.
 * \see SearchController::search()
 */
int LuaSearch::search ( lua_State* L )
{
    bool code      = true;
    bool caseS     = false;
    bool regex     = false;
    bool comment   = false;
    bool collation = false;
    bool linestring= false;

    // take the flags from the stack
    // if no flags given only code will be searched
    QString flags ( lua_tostring(L, 3) );
    QList<bool> lvls;

    if ( !flags.isEmpty () )
    {
        // search codepoint
        if ( !flags.contains ( "c" ) )
        {
            code = false;
        }
        // search collations ("weight")
        if ( flags.contains ( "w" ) )
        {
            collation = true;
        }
        // search comments ("description")
        if ( flags.contains ( "d" ) )
        {
            comment = true;
        }
        // Regular expression
        if ( flags.contains ( "r" ) )
        {
            regex = true;
        }
        // linestring
        if (flags.contains("l")) {
			linestring = true;
		}
        // caseSensitive description search
        if ( flags.contains ( "D", Qt::CaseSensitive ) )
        {
            comment = true;
            caseS = true;
        }
        lvls << flags.contains ( "l1" );
        lvls << flags.contains ( "l2" );
        lvls << flags.contains ( "l3" );
        lvls << flags.contains ( "l4" );

        // make sure collation is true, when lvls are given
        if ( !flags.contains ( "w" ) && lvls.contains ( true ) )
        {
            collation = true;
        }
    }

    searchControl_->search ( QString ( lua_tostring(L, 2) ), code, collation,
                             comment,linestring, caseS, regex, lvls );
    searchControl_->next ();
    return 0;
}

/**
 * \brief Next found letter.
 *
 * Select and shows the next found letter.
 * \see SearchController::next()
 */
int LuaSearch::next ( lua_State* /*L*/)
{
    searchControl_->next ();
    return 0;
}

/**
 * \brief Previous found letter.
 *
 * Select and shows the previous found letter.
 * \see SearchController::previous()
 */
int LuaSearch::previous ( lua_State* /*L*/)
{
    searchControl_->previous ();
    return 0;
}

/**
 * \brief Select all found letters.
 *
 * Select all found letters in the visible table.
 * \see SearchController::selectAll()
 */
int LuaSearch::selectAll ( lua_State* /*L*/)
{
    searchControl_->selectAll ();
    return 0;
}

/**
 * \brief Adds all found letters to the visible table.
 *
 * \return Returns number of results.
 * \see SearchController::addItems()
 */
int LuaSearch::addItems ( lua_State* /*L*/)
{
    searchControl_->addItems ();
    return 0;
}

/**
 * \brief Searches in the Unicode table.
 *
 * This method searches for letters in the Unicode table. It takes two
 * arguments from the Lua interpreter. The first argument is the search
 * string or regular expression. The second argument are the search flags.
 * The search flags are:<br>
 * 'c' Codepoint<br>
 * 'w' Collation<br>
 * 'd' Comment<br>
 * 'r' Regular Expression<br>
 * 'D' Case sensitive<br>
 * 'l1-4' Collation level 1-4<br>
 * \n
 * If the case sensitive flag is set, the comment flag is set, too. And
 * if a collation level flag is set, the collation flag is set, too.
 * After the search() method from SearchController is called, the next() method is
 * called to select and show the first found letter.
 *
 * \param L Lua interpreter state.
 * \return Returns number of results.
 * \see SearchController::searchTable().
 *
 */
int LuaSearch::searchTable ( lua_State* L )
{
    bool code      = true;
    bool caseS     = false;
    bool regex     = false;
    bool comment   = false;
    bool collation = false;
    bool linestring= false;

    // take the flags from the stack
    // if no flags given only code will be searched
    QString flags ( lua_tostring(L, 3) );
    QList<bool> lvls;

    // prepare list
    for ( int i = 0; i < 4; ++i )
    {
        lvls.append ( false );
    }

    if ( !flags.isEmpty () )
    {
        // search codepoint
        if ( !flags.contains ( "c" ) )
        {
            code = false;
        }
        // search collations ("weight")
        if ( flags.contains ( "w" ) )
        {
            collation = true;
        }
        // search comments ("description")
        if ( flags.contains ( "d" ) )
        {
            comment = true;
        }
        // Regular expression
        if ( flags.contains ( "r" ) )
        {
            regex = true;
        }
		// Linestring
		if (flags.contains("l")) {
			linestring = true;
		}
        // caseSensitive description search
        if ( flags.contains ( "D", Qt::CaseSensitive ) )
        {
            comment = true;
            caseS = true;
        }
        // search level1
        if ( flags.contains ( "l1" ) )
        {
            lvls.replace ( 0, true );
        }
        // search level2
        if ( flags.contains ( "l2" ) )
        {
            lvls.replace ( 1, true );
        }
        // search level3
        if ( flags.contains ( "l3" ) )
        {
            lvls.replace ( 2, true );
        }
        // search level4
        if ( flags.contains ( "l4" ) )
        {
            lvls.replace ( 3, true );
        }
    }

    searchControl_->searchTable ( QString ( lua_tostring(L, 2) ), code,
                                  collation, comment, linestring, caseS, regex, lvls );
    return 0;
}

/**
 * \brief Returns the number of found letters.
 *
 * \param L Lua interpreter state.
 * \return Return the number of results.
 * \see SearchController::numberOfHits().
 */
int LuaSearch::numberOfHits ( lua_State* L )
{
    lua_pushinteger ( L, searchControl_->numberOfHits () );
    return 1;
}

/**
 * \brief Checks if there are more hits.
 *
 * If the end of found letter list is not reached, Lua gets the
 * value true. Otherwise false.
 *
 * \param L Lua interpreter state.
 * \return Return the number of results.
 * \see SearchController::hasNext().
 */
int LuaSearch::hasNext ( lua_State* L )
{
    lua_pushboolean ( L, searchControl_->hasNext () );
    return 1;
}

/**
 * \brief Checks if there are previous hits.
 *
 * If the beginning of found letter list is not reached, Lua gets the
 * value true. Otherwise false.
 *
 * \param L Lua interpreter state.
 * \return Return number of results.
 * \see SearchController::hasPrevious().
 */
int LuaSearch::hasPrevious ( lua_State* L )
{
    lua_pushboolean ( L, searchControl_->hasPrevious () );
    return 1;
}
